import { QueryKey, UseQueryOptions, UseMutationOptions, UseMutationResult, UseQueryResult, UseSuspenseQueryOptions } from '@tanstack/react-query';
import * as react from 'react';

/** Represents the static methods from the generated service client. */
type ServiceMethod<I, O> = (req: I, initReq: RequestInitWithPathPrefix) => Promise<O>;
/**
 * Extends the standard `RequestInit` accepted by `fetch()` with a `pathPrefix`
 *property that is used by the generated gateway methods to prefix the URL.
 */
interface RequestInitWithPathPrefix extends RequestInit {
    pathPrefix?: string;
}
/** Represents an error returned by calling a service method. */
type ServiceError = Error | ErrorResponse;
/** Represents a function that handles errors returned from a service method. */
type OnErrorHandler<O> = (error: Error | ErrorResponse) => O | null;
/** Represents the standard error response returned from the server. */
interface ErrorResponse {
    codeName?: string;
    code: number;
    message: string;
    details: unknown[];
}
/**
 * Represents the `useServiceQuery` options. This is the same as
 * `UseQueryOptions` except that `queryFn` is handled internally, so must not
 * be provided and `queryKey` becomes optional.
 * Additionally an `onError` handler can be provided to provide customized error
 * handling.
 */
type UseServiceQueryOptions<M extends ServiceMethod<Parameters<M>[0], ReturnType<M>>, Q extends QueryKey> = Omit<UseQueryOptions<Awaited<ReturnType<M>>, ServiceError>, 'queryFn' | 'queryKey'> & {
    onError?: OnErrorHandler<ReturnType<M>>;
} & {
    queryKey?: Q;
};
/** Represents the `useServiceMutation` options. This is the same as
 * `UseMutationOptions` except that `mutationFn` is handled internally, so must
 * not be provided.
 */
type UseServiceMutationOptions<M extends ServiceMethod<Parameters<M>[0], ReturnType<M>>, C = unknown> = Omit<UseMutationOptions<Awaited<ReturnType<M>>, ServiceError, Parameters<M>[0], C>, 'mutationFn'>;
/** Returns true if the error is an error response. */
declare function isErrorResponse(error: Error | ErrorResponse): error is ErrorResponse;

/**
 * Returns a default query key for a service method and request object. The key
 * structure should not be depended on and considered internal. It is subject to
 * change.
 */
declare function queryKey<M extends ServiceMethod<Parameters<M>[0], Awaited<ReturnType<M>>>>(method: M, req?: Parameters<M>[0]): [string] | [string, Parameters<M>[0]];

/**
 * Represents the context object for injecting request options.
 *
 * Example:
 *
 *    <ServiceContext.Provider value={options}>
 *      {children}
 *    </ServiceContext.Provider>
 *
 * Where options can include `pathPrefix` or any of the options in `RequestInit`.
 *
 * https://microsoft.github.io/PowerBI-JavaScript/interfaces/_node_modules_typedoc_node_modules_typescript_lib_lib_dom_d_.requestinit.html
 */
declare const ServiceContext: react.Context<RequestInitWithPathPrefix>;
/** useServiceContext returns the current request options from the context. */
declare const useServiceContext: () => RequestInitWithPathPrefix;

interface SideEffectOptions<TSourceReq, TSourceResp, TTargetReq, TTargetResp> {
    /**
     * Maps the querykey for the source request to the target request. If not
     * provided, the source key is used. The value is passed like so:
     *
     *     queryKey(someMethod, mapKey(req));
     */
    mapKey?: (update: TSourceReq) => TTargetReq;
    /**
     * Optimistically patches the target query's data with the update from from
     * the source query. For example, an update to a user record might
     * optimistically update the user record and a list of users.
     */
    patchFn?: (oldData: TTargetResp, update: TSourceReq) => TTargetResp;
    /**
     * Updates the target query's data with the result from the source query. This
     * is helpful if the mutation had side effects not captured in the request
     * payload.
     */
    updateFn?: (oldData: TTargetResp, result: TSourceResp) => TTargetResp;
    /**
     * If specified, the target query will be invalidated on mutation. This is
     * useful if there are known side-effects but the response from the source
     * query does not capture them. 'true' will trigger the default refetch
     * behavior of 'active', while 'active', 'inactive', 'all', and 'none' can be
     * used to specify other behaviors per `invalidateQueries`.
     * 'remove' will simply remove the query from the cache.
     */
    invalidate?: boolean | 'active' | 'inactive' | 'all' | 'remove' | 'none';
}
type SideEffectContext = Record<string, unknown>;
/**
 * Represents a subset of MutationOptions from React Query.
 */
interface SideEffectHandlers<TSourceReq, TSourceResp> {
    onMutate: (req: TSourceReq) => Promise<SideEffectContext> | SideEffectContext;
    onSuccess: (result: TSourceResp, req: TSourceReq, _context: SideEffectContext) => Promise<void> | void;
    onError: (error: ServiceError, req: TSourceReq, context: SideEffectContext) => Promise<void> | void;
}
/**
 * Specifies a side effect handler for a mutation. Side effects can
 * optimistically patch the data associated with a query, update the data, or
 * invalidate the query entirely.
 *
 * Consider an `updateUser` endpoint that allows for partial updates to a user
 * record. The shape of the response mirrors the `readUser` endpoint, as does
 * the query key.
 *
 *     useServiceMutation(updateUser, {
 *       retryDelay: 1000,
 *       ...chain(
 *         sideEffect(readUser, {
 *           patch: (oldUser, update) => ({ ...oldUser, ...update } }),
 *           update: (newUser) => newUser,
 *         }),
 *         sideEffect(listUsers, {
 *           mapKey: ({ workspaceId }) => workspaceId,
 *           invalidate: true,
 *         }),
 *       ),
 *     });
 *
 */
declare function sideEffect<TSourceReq, TSourceResp, TTargetReq, TTargetResp>(m: ServiceMethod<TTargetReq, TTargetResp>, options: SideEffectOptions<TSourceReq, TSourceResp, TTargetReq, TTargetResp>): SideEffectHandlers<TSourceReq, TSourceResp>;
declare function chainSideEffects<TSourceReq, TSourceResp>(...mutations: SideEffectHandlers<TSourceReq, TSourceResp>[]): {
    onMutate: (req: TSourceReq) => Promise<{}>;
    onSuccess: (result: TSourceResp, req: TSourceReq, context: any) => Promise<void>;
    onError: (error: ServiceError, req: TSourceReq, context: any) => Promise<void>;
};
declare function patchFn<TSourceReq, TTargetResp>(oldData: TTargetResp, update: TSourceReq): TTargetResp;
declare function updateFn<TSourceResp, TTargetResp>(_oldData: TTargetResp, result: TSourceResp): TTargetResp;

/**
 * Wraps `useMutation` from react-query, pulling request configuration from
 * Context and making it easier to call generated service clients.
 */
declare function useServiceMutation<M extends ServiceMethod<Parameters<M>[0], Awaited<ReturnType<M>>>, C = unknown>(method: M, options?: UseServiceMutationOptions<M, C>): UseMutationResult<Awaited<ReturnType<M>>, ServiceError, Partial<Parameters<M>[0]>, C>;

/**
 *  Wraps `useQuery` from react-query, pulling request configuration from context
 * and making it easier to call generated service clients.
 */
declare function useServiceQuery<M extends ServiceMethod<Parameters<M>[0], Awaited<ReturnType<M>>>, Q extends QueryKey>(method: M, req: Parameters<M>[0], options?: UseServiceQueryOptions<M, Q>): UseQueryResult<Awaited<ReturnType<M>>, ServiceError>;
/**
 * Returns the options object for `useQuery` based on the service method. Can
 * be used with `useSuspenseQuery` for data loading.
 */
declare function queryOptions<M extends ServiceMethod<Parameters<M>[0], Awaited<ReturnType<M>>>, Q extends QueryKey>(method: M, req: Parameters<M>[0], reqInit?: RequestInitWithPathPrefix, options?: UseServiceQueryOptions<M, Q>): UseSuspenseQueryOptions<Awaited<ReturnType<M>>, ServiceError>;

export { type ErrorResponse, type OnErrorHandler, type RequestInitWithPathPrefix, ServiceContext, type ServiceError, type ServiceMethod, type SideEffectHandlers, type SideEffectOptions, type UseServiceMutationOptions, type UseServiceQueryOptions, chainSideEffects, isErrorResponse, patchFn, queryKey, queryOptions, sideEffect, updateFn, useServiceContext, useServiceMutation, useServiceQuery };
