// src/queryKey.ts
function queryKey(method, req) {
  if (req === void 0) return [method.name];
  return [method.name, req];
}

// src/serviceContext.ts
import { createContext, useContext } from "react";
var ServiceContext = createContext({});
var useServiceContext = () => {
  return useContext(ServiceContext);
};

// src/sideEffect.ts
import { useQueryClient } from "@tanstack/react-query";
function sideEffect(m, options) {
  const queryClient = useQueryClient();
  const { mapKey = (x) => x, patchFn: patch, updateFn: update, invalidate } = options;
  return {
    onMutate: async (req) => {
      const key = queryKey(m, mapKey ? mapKey(req) : void 0);
      await queryClient.cancelQueries({ queryKey: key });
      let originalData;
      queryClient.setQueryData(key, (oldData) => {
        originalData = oldData;
        return patch ? patch(oldData, req) : oldData;
      });
      return { [key.join("-")]: originalData };
    },
    onSuccess: async (result, req, _context) => {
      const key = queryKey(m, mapKey ? mapKey(req) : void 0);
      if (update) {
        queryClient.setQueryData(key, (oldData) => update(oldData, result));
      }
      if (invalidate === "remove") {
        queryClient.removeQueries({ queryKey: key });
      } else if (invalidate) {
        await queryClient.invalidateQueries({
          queryKey: key,
          refetchType: typeof invalidate == "string" ? invalidate : "active"
        });
      }
    },
    onError: (_error, req, context) => {
      const key = queryKey(m, mapKey ? mapKey(req) : void 0);
      const originalData = context[key.join("-")];
      queryClient.setQueryData(key, originalData);
    }
  };
}
function chainSideEffects(...mutations) {
  return {
    onMutate: async (req) => {
      const context = {};
      for (const mutation of mutations) {
        Object.assign(context, await mutation?.onMutate(req));
      }
      return context;
    },
    onSuccess: async (result, req, context) => {
      for (const mutation of mutations) {
        await mutation?.onSuccess(result, req, context);
      }
    },
    onError: async (error, req, context) => {
      for (const mutation of mutations) {
        await mutation?.onError(error, req, context);
      }
    }
  };
}
function patchFn(oldData, update) {
  return { ...oldData, ...update };
}
function updateFn(_oldData, result) {
  return result;
}

// src/types.ts
function isErrorResponse(error) {
  return error.code !== void 0;
}

// src/useServiceMutation.ts
import { useMutation } from "@tanstack/react-query";
import { useContext as useContext2 } from "react";
function useServiceMutation(method, options) {
  const reqCtx = useContext2(ServiceContext);
  return useMutation({
    ...options,
    mutationFn: (req) => {
      const resp = method(req, {
        ...reqCtx,
        headers: {
          "Content-Type": "application/json",
          ...reqCtx.headers
        }
      });
      return resp;
    }
  });
}

// src/useServiceQuery.ts
import { useQuery } from "@tanstack/react-query";
import { useContext as useContext3 } from "react";
function useServiceQuery(method, req, options) {
  const reqCtx = useContext3(ServiceContext);
  return useQuery(queryOptions(method, req, reqCtx, options));
}
function queryOptions(method, req, reqInit, options) {
  return {
    ...options,
    queryKey: options?.queryKey ?? queryKey(method, req),
    queryFn: () => {
      const resp = method(req, {
        ...reqInit,
        headers: {
          "Content-Type": "application/json",
          ...reqInit?.headers
        }
      });
      if (options?.onError) {
        return resp.catch(options.onError);
      }
      return resp;
    }
  };
}
export {
  ServiceContext,
  chainSideEffects,
  isErrorResponse,
  patchFn,
  queryKey,
  queryOptions,
  sideEffect,
  updateFn,
  useServiceContext,
  useServiceMutation,
  useServiceQuery
};
//# sourceMappingURL=index.js.map